/****************************************************************************
** Meta object code from reading C++ file 'ProcessRunner.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../print-manager-6.7.2/src/libkcups/ProcessRunner.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ProcessRunner.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13ProcessRunnerE_t {};
} // unnamed namespace

template <> constexpr inline auto ProcessRunner::qt_create_metaobjectdata<qt_meta_tag_ZN13ProcessRunnerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ProcessRunner",
        "QML.Element",
        "auto",
        "QML.Singleton",
        "true",
        "configurePrinter",
        "",
        "printerName",
        "openPrintQueue",
        "addPrinter",
        "startupId",
        "addClass",
        "openKCM",
        "args",
        "kcmConfigurePrinter"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'configurePrinter'
        QtMocHelpers::SlotData<void(const QString &)>(5, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
        // Slot 'openPrintQueue'
        QtMocHelpers::SlotData<void(const QString &)>(8, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
        // Slot 'addPrinter'
        QtMocHelpers::SlotData<void(const QByteArray)>(9, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 10 },
        }}),
        // Slot 'addPrinter'
        QtMocHelpers::SlotData<void()>(9, 6, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'addClass'
        QtMocHelpers::SlotData<void(const QByteArray)>(11, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 10 },
        }}),
        // Slot 'addClass'
        QtMocHelpers::SlotData<void()>(11, 6, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'openKCM'
        QtMocHelpers::SlotData<void(const QStringList &, const QByteArray)>(12, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 13 }, { QMetaType::QByteArray, 10 },
        }}),
        // Slot 'openKCM'
        QtMocHelpers::SlotData<void(const QStringList &)>(12, 6, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QStringList, 13 },
        }}),
        // Slot 'openKCM'
        QtMocHelpers::SlotData<void()>(12, 6, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'kcmConfigurePrinter'
        QtMocHelpers::SlotData<void(const QString &, const QByteArray)>(14, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 }, { QMetaType::QByteArray, 10 },
        }}),
        // Slot 'kcmConfigurePrinter'
        QtMocHelpers::SlotData<void(const QString &)>(14, 6, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<ProcessRunner, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject ProcessRunner::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13ProcessRunnerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13ProcessRunnerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13ProcessRunnerE_t>.metaTypes,
    nullptr
} };

void ProcessRunner::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ProcessRunner *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->configurePrinter((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->openPrintQueue((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->addPrinter((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 3: _t->addPrinter(); break;
        case 4: _t->addClass((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 5: _t->addClass(); break;
        case 6: _t->openKCM((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[2]))); break;
        case 7: _t->openKCM((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 8: _t->openKCM(); break;
        case 9: _t->kcmConfigurePrinter((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[2]))); break;
        case 10: _t->kcmConfigurePrinter((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *ProcessRunner::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ProcessRunner::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13ProcessRunnerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int ProcessRunner::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 11;
    }
    return _id;
}
QT_WARNING_POP
